/*
 * Copyright or © or Copr. Fabien Michel, Olivier Gutknecht, Jacques Ferber (1997)

fmichel@lirmm.fr
olg@no-distance.net
ferber@lirmm.fr

This software is a computer program whose purpose is to 
provide a lightweight Java library for designing and simulating Multi-Agent Systems (MAS).

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
 */
package madkit.simulation.model;

import java.lang.reflect.Field;

import javax.swing.BoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import madkit.gui.SwingUtil;
import madkit.kernel.AbstractAgent;

/**
 * Prototype class that will be used to encapsulate simulation models
 * in a near future.
 * 
 * @author Fabien Michel
 * @since MadKit 5.0.0.19
 * @version 0.9
 * 
 */
public class AbstractModel extends AbstractAgent {

	/**
	 * Returns a panel containing sliders operating on all the
	 * {@link BoundedRangeModel} fields which are found in this model.
	 * 
	 * @param label the name of the panel
	 * @return a panel containing sliders
	 */
	public JPanel getParametersView(String label) {
		return getParametersView(label, this);
	}

	/**
	 * Returns a panel containing sliders operating on all the
	 * {@link BoundedRangeModel} fields which are found in this model.
	 * 
	 * @param label the name of the panel
	 * @return a panel containing sliders
	 */
	@SuppressWarnings("unchecked")
	public JPanel getParametersView(String label, AbstractAgent onInstance) {
		final JPanel all = new JPanel();
		all.setBorder(new TitledBorder(label));
		Class<? extends AbstractAgent> agentClass = onInstance.getClass();
		while (true) {
			for (Field f : agentClass.getDeclaredFields()) {
				if (BoundedRangeModel.class.isAssignableFrom(f.getType())) {
					f.setAccessible(true);
					try {
						all.add(SwingUtil.createSliderPanel(
								(BoundedRangeModel) f.get(onInstance), f.getName()));
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}
			agentClass = (Class<? extends AbstractAgent>) agentClass.getSuperclass();
			if(agentClass == AbstractAgent.class)
				break;
		}
		return all;
	}

}
